import Foundation
import Flutter
import FBAudienceNetwork


class SwiftFacebookAudienceNetworkNativeAdFactory: NSObject, FlutterPlatformViewFactory {
    let registrar: FlutterPluginRegistrar
    init(_registrar: FlutterPluginRegistrar) {
        print("NativeAd > Factory register")
        
        registrar = _registrar
        super.init()
    }
    
    func createArgsCodec() -> FlutterMessageCodec & NSObjectProtocol {
        print("NativeAd > Factory createArgsCodec")
        
        return FlutterStandardMessageCodec.sharedInstance()
    }
    
    func create(withFrame frame: CGRect, viewIdentifier viewId: Int64, arguments args: Any?) -> FlutterPlatformView {
        print("NativeAd > Factory create")
        
        return SwiftFacebookAudienceNetworkNativeAdView(_frame: frame,
                                                _viewId: viewId,
                                                _params: args as? Dictionary<String, Any> ?? nil,
                                                _registrar: registrar)
    }
}


class SwiftFacebookAudienceNetworkNativeAdView: NSObject, FlutterPlatformView, FBNativeAdDelegate {
    private let frame: CGRect
    private let viewId: Int64
    private let registrar: FlutterPluginRegistrar
    private let params: [String: Any]
    private let channel: FlutterMethodChannel
    
    var mainView: UIView!
    var nativeAd: FBNativeAd!
    var nativeAdViewAttributes: FBNativeAdViewAttributes!
    
    init(_frame: CGRect,
         _viewId: Int64,
         _params: [String: Any]?,
         _registrar: FlutterPluginRegistrar) {
        
        frame = _frame
        viewId = _viewId
        registrar = _registrar
        params = _params!
        channel = FlutterMethodChannel(
            name: "\(FbConstant.NATIVE_AD_CHANNEL)_\(viewId)",
            binaryMessenger: registrar.messenger()
        )
        
        super.init()
        
        channel.setMethodCallHandler { [weak self] (call, result) in
            guard let `self` = self else { return }
            `self`.handle(call, result: result)
        }
        
        initView()
        initFB()
    }
    
    deinit {
        print("NativeAd > is deninit")
    }
    
    func view() -> UIView {
        return mainView
    }
    
    
    /**
     * handle
     */
    func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "initialization":
            result(true)
        case "init":
            result(true)
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    
    
    /**
     * initView
     **/
    func initView() {
        print("NativeAd > init initView")
        
        self.mainView = UIView(frame: self.frame)
        self.mainView.backgroundColor = UIColor.white
    }
    
    
    /**
     * init Ad
     **/
    func initFB() {
        print("NativeAd > init initFB")
        
        initNativeAd()
        initNativeAdViewAttributes()
    }
    
    
    /**
     * load
     **/
    func initNativeAd() {
        print("NativeAd > init initNativeAd")
        
        let existsId: Bool = (self.params["id"] != nil) ? true : false
        
        if (existsId) {
            let valueId: String = self.params["id"] as! String
            self.nativeAd = FBNativeAd(placementID: valueId)
            self.nativeAd.delegate = self
            self.nativeAd.loadAd()
        }
    }
    
    func initNativeAdViewAttributes() {
        print("NativeAd > initNativeAdViewAttributes")
        
        let buttonColor: String = self.params["button_border_color"] as! String
        let buttonTitleColor: String = self.params["button_title_color"] as! String
        let bgColor: String = self.params["bg_color"] as! String
        let titleColor: String = self.params["title_color"] as! String
        let descColor: String = self.params["desc_color"] as! String
        
        let buttonColorUIColor = UIColor.init(hexString: buttonColor)
        let buttonTitleColorUIColor = UIColor.init(hexString: buttonTitleColor)
        let bgColorUIColor = UIColor.init(hexString: bgColor)
        let titleColorUIColor = UIColor.init(hexString: titleColor)
        let descColorUIColor = UIColor.init(hexString: descColor)
        
        self.nativeAdViewAttributes = FBNativeAdViewAttributes()
        self.nativeAdViewAttributes.buttonColor = buttonColorUIColor
        self.nativeAdViewAttributes.buttonTitleColor = buttonTitleColorUIColor
        self.nativeAdViewAttributes.backgroundColor = bgColorUIColor
        self.nativeAdViewAttributes.titleColor = titleColorUIColor
        self.nativeAdViewAttributes.descriptionColor = descColorUIColor
        
        self.nativeAdViewAttributes.titleFont = UIFont(name: "Noteworthy", size: 15.0)
        self.nativeAdViewAttributes.buttonTitleFont = UIFont(name: "Futura", size: 12.0)
    }
    
    /**
     * using native ad
     */
    func regNativeAdViewTemplate() {
        print("NativeAd > regNativeAdViewTemplate")
        let height: CGFloat = (self.params["height"] != nil) ? self.params["height"] as! CGFloat : 250.0
        let nativeAdView = FBNativeAdView.init(nativeAd: self.nativeAd, with: FBNativeAdViewType.dynamic, with: self.nativeAdViewAttributes)
        nativeAdView.frame = CGRect(x: 0.0, y: 0.0, width: UIScreen.main.bounds.size.width, height: height)
        self.mainView.addSubview(nativeAdView)
        self.mainView.layoutIfNeeded()
    }
    
    
    /**
     Sent when an FBNativeAd has been successfully loaded.
     
     @param nativeAd An FBNativeAd object sending the message.
     */
    func nativeAdDidLoad(_ nativeAd: FBNativeAd) {
        print("NativeAd > nativeAdDidLoad")
        self.nativeAd = nativeAd
        regNativeAdViewTemplate()
        
        let placement_id: String = nativeAd.placementID
        let invalidated: Bool = nativeAd.isAdValid
        let arg: [String: Any] = [
            FbConstant.PLACEMENT_ID_ARG: placement_id,
            FbConstant.INVALIDATED_ARG: invalidated,
        ]
        self.channel.invokeMethod(FbConstant.LOADED_METHOD, arguments: arg)
    }
    
    /**
     Sent when an FBNativeAd has succesfully downloaded all media
     */
    func nativeAdDidDownloadMedia(_ nativeAd: FBNativeAd) {
        print("NativeAd > nativeAdDidDownloadMedia")
    }
    
    /**
     Sent immediately before the impression of an FBNativeAd object will be logged.
     
     @param nativeAd An FBNativeAd object sending the message.
     */
    func nativeAdWillLogImpression(_ nativeAd: FBNativeAd) {
        print("NativeAd > nativeAdWillLogImpression")
        
        let placement_id: String = nativeAd.placementID
        let invalidated: Bool = nativeAd.isAdValid
        let arg: [String: Any] = [
            FbConstant.PLACEMENT_ID_ARG: placement_id,
            FbConstant.INVALIDATED_ARG: invalidated,
        ]
        self.channel.invokeMethod(FbConstant.LOGGING_IMPRESSION_METHOD, arguments: arg)
    }
    
    /**
     Sent when an FBNativeAd is failed to load.
     
     @param nativeAd An FBNativeAd object sending the message.
     @param error An error object containing details of the error.
     */
    func nativeAd(_ nativeAd: FBNativeAd, didFailWithError error: Error) {
        print("NativeAd > nativeAd %s", error)
        
        let placement_id: String = nativeAd.placementID
        let invalidated: Bool = nativeAd.isAdValid
        let errorStr: String = error as! String
        let arg: [String: Any] = [
            FbConstant.PLACEMENT_ID_ARG: placement_id,
            FbConstant.INVALIDATED_ARG: invalidated,
            FbConstant.ERROR_ARG:errorStr
        ]
        self.channel.invokeMethod(FbConstant.ERROR_METHOD, arguments: arg)
    }
    
    /**
     Sent after an ad has been clicked by the person.
     
     @param nativeAd An FBNativeAd object sending the message.
     */
    func nativeAdDidClick(_ nativeAd: FBNativeAd) {
        print("NativeAd > nativeAdDidClick")
        
        let placement_id: String = nativeAd.placementID
        let invalidated: Bool = nativeAd.isAdValid
        let arg: [String: Any] = [
            FbConstant.PLACEMENT_ID_ARG: placement_id,
            FbConstant.INVALIDATED_ARG: invalidated,
        ]
        self.channel.invokeMethod(FbConstant.CLICKED_METHOD, arguments: arg)
    }
}



